package sun.security.krb5;

import sun.security.krb5.internal.Krb5;
import java.net.UnknownHostException;
import java.util.Arrays;
import sun.security.krb5.internal.APOptions;
import sun.security.krb5.internal.KDCReqBody;
import sun.security.krb5.internal.crypto.Nonce;
import sun.security.krb5.internal.crypto.EType;
import sun.security.krb5.internal.AuthorizationData;
import sun.security.krb5.internal.HostAddresses;
import java.io.IOException;
import sun.security.krb5.internal.PAData;
import sun.security.krb5.internal.KDCOptions;
import sun.security.krb5.internal.Ticket;
import sun.security.krb5.internal.KerberosTime;
import sun.security.krb5.internal.TGSReq;

public class KrbTgsReq
{
    private PrincipalName princName;
    private PrincipalName clientAlias;
    private PrincipalName servName;
    private PrincipalName serverAlias;
    private TGSReq tgsReqMessg;
    private KerberosTime ctime;
    private Ticket secondTicket;
    private boolean useSubkey;
    EncryptionKey tgsReqKey;
    private static final boolean DEBUG;
    private byte[] obuf;
    private byte[] ibuf;
    
    public KrbTgsReq(KDCOptions kdcOptions, Credentials credentials, PrincipalName principalName, PrincipalName principalName2, PrincipalName principalName3, PrincipalName principalName4, Ticket[] array, PAData[] array2) throws KrbException, IOException {
        this(kdcOptions, credentials, principalName, principalName2, principalName3, principalName4, null, null, null, null, null, null, array, null, array2);
    }
    
    KrbTgsReq(KDCOptions kdcOptions, Credentials credentials, PrincipalName principalName, PrincipalName principalName2, KerberosTime kerberosTime, KerberosTime kerberosTime2, KerberosTime kerberosTime3, int[] array, HostAddresses hostAddresses, AuthorizationData authorizationData, Ticket[] array2, EncryptionKey encryptionKey) throws KrbException, IOException {
        this(kdcOptions, credentials, credentials.getClient(), credentials.getClientAlias(), principalName, principalName2, kerberosTime, kerberosTime2, kerberosTime3, array, hostAddresses, authorizationData, array2, encryptionKey, (PAData[])null);
    }
    
    private KrbTgsReq(KDCOptions kdcOptions, Credentials credentials, PrincipalName princName, PrincipalName clientAlias, PrincipalName servName, PrincipalName serverAlias, KerberosTime kerberosTime, KerberosTime kerberosTime2, KerberosTime kerberosTime3, int[] array, HostAddresses hostAddresses, AuthorizationData authorizationData, Ticket[] array2, EncryptionKey encryptionKey, PAData[] array3) throws KrbException, IOException {
        this.secondTicket = null;
        this.useSubkey = false;
        this.princName = princName;
        this.clientAlias = clientAlias;
        this.servName = servName;
        this.serverAlias = serverAlias;
        this.ctime = KerberosTime.now();
        if (kdcOptions.get(1) && !credentials.flags.get(1)) {
            kdcOptions.set(1, false);
        }
        if (kdcOptions.get(2) && !credentials.flags.get(1)) {
            throw new KrbException(101);
        }
        if (kdcOptions.get(3) && !credentials.flags.get(3)) {
            throw new KrbException(101);
        }
        if (kdcOptions.get(4) && !credentials.flags.get(3)) {
            throw new KrbException(101);
        }
        if (kdcOptions.get(5) && !credentials.flags.get(5)) {
            throw new KrbException(101);
        }
        if (kdcOptions.get(8) && !credentials.flags.get(8)) {
            throw new KrbException(101);
        }
        if (kdcOptions.get(6)) {
            if (!credentials.flags.get(6)) {
                throw new KrbException(101);
            }
        }
        else if (kerberosTime != null) {
            kerberosTime = null;
        }
        if (kdcOptions.get(8)) {
            if (!credentials.flags.get(8)) {
                throw new KrbException(101);
            }
        }
        else if (kerberosTime3 != null) {
            kerberosTime3 = null;
        }
        if (kdcOptions.get(28) || kdcOptions.get(14)) {
            if (array2 == null) {
                throw new KrbException(101);
            }
            this.secondTicket = array2[0];
        }
        else if (array2 != null) {
            array2 = null;
        }
        this.tgsReqMessg = this.createRequest(kdcOptions, credentials.ticket, credentials.key, this.ctime, this.princName, this.servName, kerberosTime, kerberosTime2, kerberosTime3, array, hostAddresses, authorizationData, array2, encryptionKey, array3);
        this.obuf = this.tgsReqMessg.asn1Encode();
        if (credentials.flags.get(2)) {
            kdcOptions.set(2, true);
        }
        return;
    }
    
    public void send() throws IOException, KrbException {
        String realmString = null;
        if (this.servName != null) {
            realmString = this.servName.getRealmString();
        }
        this.ibuf = new KdcComm(realmString).send(this.obuf);
    }
    
    public KrbTgsRep getReply() throws KrbException, IOException {
        return new KrbTgsRep(this.ibuf, this);
    }
    
    public Credentials sendAndGetCreds() throws IOException, KrbException {
        this.send();
        return this.getReply().getCreds();
    }
    
    KerberosTime getCtime() {
        return this.ctime;
    }
    
    private TGSReq createRequest(KDCOptions new_kdcOptions, Ticket ticket, EncryptionKey tgsReqKey, KerberosTime kerberosTime, PrincipalName new_cname, PrincipalName new_sname, KerberosTime new_from, KerberosTime kerberosTime2, KerberosTime new_rtime, int[] array, HostAddresses new_addresses, AuthorizationData authorizationData, Ticket[] new_additionalTickets, EncryptionKey tgsReqKey2, PAData[] original) throws IOException, KrbException, UnknownHostException {
        KerberosTime new_till;
        if (kerberosTime2 == null) {
            new_till = new KerberosTime(0L);
        }
        else {
            new_till = kerberosTime2;
        }
        this.tgsReqKey = tgsReqKey;
        int[] defaults;
        if (array == null) {
            defaults = EType.getDefaults("default_tgs_enctypes");
        }
        else {
            defaults = array;
        }
        EncryptionKey encryptionKey = null;
        Object new_encAuthorizationData = null;
        if (authorizationData != null) {
            byte[] asn1Encode = authorizationData.asn1Encode();
            if (tgsReqKey2 != null) {
                encryptionKey = tgsReqKey2;
                this.tgsReqKey = tgsReqKey2;
                this.useSubkey = true;
                new_encAuthorizationData = new EncryptedData(encryptionKey, asn1Encode, 5);
            }
            else {
                new_encAuthorizationData = new EncryptedData(tgsReqKey, asn1Encode, 4);
            }
        }
        KDCReqBody new_reqBody = new KDCReqBody(new_kdcOptions, new_cname, new_sname, new_from, new_till, new_rtime, Nonce.value(), defaults, new_addresses, new_encAuthorizationData, new_additionalTickets);
        PAData o = new PAData(1, new KrbApReq(new APOptions(), ticket, tgsReqKey, new_cname, new Checksum(Checksum.CKSUMTYPE_DEFAULT, new_reqBody.asn1Encode(12), tgsReqKey, 6), kerberosTime, encryptionKey, null, null).getMessage());
        PAData[] new_pAData;
        if (original != null) {
            new_pAData = (PAData[])Arrays.copyOf(original, original.length + 1);
            new_pAData[original.length] = o;
        }
        else {
            new_pAData = new PAData[] { o };
        }
        return new TGSReq(new_pAData, new_reqBody);
    }
    
    TGSReq getMessage() {
        return this.tgsReqMessg;
    }
    
    Ticket getSecondTicket() {
        return this.secondTicket;
    }
    
    PrincipalName getClientAlias() {
        return this.clientAlias;
    }
    
    PrincipalName getServerAlias() {
        return this.serverAlias;
    }
    
    private static void debug(String s) {
    }
    
    boolean usedSubkey() {
        return this.useSubkey;
    }
    
    static {
        DEBUG = Krb5.DEBUG;
    }
}
